@@ -156,9 +156,11 @@ class Board():
         Returns:
             color: The color of the king that is in check or False
         """
+        if color:
+            color = colors.BLACK if color == colors.WHITE else colors.WHITE
         pieces = list(self.piece_list(self.squares, color))
         for piece in pieces:
-            moves = [BoardMove(self, i.text) for i in piece.allowed_moves()]
+            moves = piece.allowed_moves(board=self, check_checking=True)
             for move in moves:
                 if not move:
                     continue
@@ -166,6 +168,9 @@ class Board():
                 if not move.destination_sq.occupant:
                     continue
 
+                if not move.valid:
+                    continue
+
                 if (isinstance(move.destination_sq.occupant, King) and
                         move.destination_sq.occupant.color != move.origin_sq.
                         occupant.color):
@@ -178,6 +183,10 @@ class Board():
 
         Returns:
             color: Color of the king in mate or False
+
+        Todo:
+            * Tests make me believe this introduces some heavy recursion shit
+            * It's also a mess so those might be related
         """
         check = self.is_check()
         if not check:
@@ -186,15 +195,14 @@ class Board():
         pieces = list(self.piece_list(self.squares, check))
         moves = []
         for i in pieces:
-            moves = moves + i.allowed_moves()
+            for move in i.allowed_moves(board=self, check_checking=True):
+                moves.append(move)
 
-        boardmoves = [BoardMove(self, i.text) for i in moves]
-
-        for move in boardmoves:
-            valid_move = move.process()
-            if valid_move:
-                if not valid_move.is_check(check):
-                    return False
+        # TODO This creates some huge loading times when a lot of pieces are
+        # there
+        for move in moves:
+            if not self.new_board(move).is_check(color=check):
+                return False
 
         return check
 
@@ -212,23 +220,22 @@ class Board():
             newboard: The new board
         """
         newboard = copy.deepcopy(self)
-        old_move = move
-        move = BoardMove(newboard, move.text)
+        old_move = BoardMove(newboard, move.text, rule_check=False)
+        move = BoardMove(newboard, move.text, rule_check=False)
 
         piece = move.origin_sq.occupant
 
         move.origin_sq.occupant = None
         if move.destination_sq.occupant:
             move.destination_sq.occupant.captured = True
-        try:
-            if old_move.enpassant:
-                o_letter = old_move.enpassant.selector[0]
-                o_number = int(old_move.enpassant.selector[1])
+        if old_move.enpassant:
+            d_letter = old_move.destination[0]
+            o_number = int(old_move.origin[1])
+            pass_sq = newboard.squares[d_letter][o_number]
+
+            pass_sq.occupant.captured = True
+            pass_sq.occupant = None
 
-                newboard.squares[o_letter][o_number].occupant.captured = True
-                newboard.squares[o_letter][o_number].occupant = None
-        except AttributeError:
-            pass
         move.destination_sq.occupant = piece
         piece.position = move.destination_sq.selector
 
@@ -278,6 +285,14 @@ class Board():
 
         return newboard
 
+    def __deepcopy__(self, memodict={}):
+        """Deepcopy."""
+        obj = Board()
+        obj.squares = copy.deepcopy(self.squares)
+        obj.__create_board = None
+
+        return obj
+
 
 class Square():
     """Represents a square on a chessboard.
@@ -308,3 +323,13 @@ class Square():
             return "[{}]".format(str(self.occupant))
         else:
             return '[ ]'
+
+    def __deepcopy__(self, memodict={}):
+        """Deepcopy."""
+        obj = Square(
+            self.selector,
+            self.color,
+            occupant=copy.deepcopy(self.occupant)
+        )
+
+        return obj
