@@ -29,14 +29,22 @@ class BoardMove(Move):
         board(Board): The boardcontext.
         origin_sq(Square): The origin square.
         destination_sq(Square): The destination square.
+        valid(Bool): Is this move considered valid.
     """
 
-    def __init__(self, board, *args, **kwargs):
+    def __init__(self, board, *args,
+                 rule_check=True, check_checking=False, **kwargs):
         """Init."""
         super().__init__(*args, **kwargs)
+
         self.board = board
         (self.origin_sq,
          self.destination_sq) = self.__get_origin_destination()
+        self.enpassant = self.__is_enpassant()
+
+        if rule_check:
+            rules = BoardRules(self, check_checking=check_checking)
+            self.valid = rules.valid
 
     def squares_between(self):
         """Get the squares between origin and destination of this move.
@@ -44,11 +52,14 @@ class BoardMove(Move):
         Yields:
             Square: The squares between origin and destination of this move.
         """
-        for selector in self.square_selectors_between():
-            c = selector[0]
-            i = int(selector[1])
+        try:
+            for selector in self.square_selectors_between():
+                c = selector[0]
+                i = int(selector[1])
 
-            yield self.board.squares[c][i]
+                yield self.board.squares[c][i]
+        except ValueError:
+            pass
 
     def castle_side(self):
         """Return the side we're castling to."""
@@ -111,6 +122,34 @@ class BoardMove(Move):
 
         return piece.color
 
+    def __is_enpassant(self):
+        """Is the move en passant."""
+        if not isinstance(self.origin_sq.occupant, pieces.Pawn):
+            return False
+
+        if not self.horizontal == 1 and not self.horizontal == -1:
+            return False
+
+        if self.destination_sq.occupant:
+            return False
+
+        d_letter = self.destination[0]
+        o_number = int(self.origin[1])
+        pass_sq = self.board.squares[d_letter][o_number]
+
+        if (isinstance(pass_sq.occupant, pieces.Pawn)
+           and pass_sq.occupant.color != self.origin_sq.occupant):
+            if pass_sq.occupant.last_move:
+                if (pass_sq.occupant.last_move.vertical == 2
+                        or pass_sq.occupant.last_move.vertical == -2):
+                    return True
+                else:
+                    return False
+            else:
+                return False
+        else:
+            return False
+
     def __get_origin_destination(self):
         """Get the origin and destination square of this move.
 
@@ -128,7 +167,7 @@ class BoardMove(Move):
 
         return (origin, destination)
 
-    def process(self):
+    def process(self, debug=False):
         """Process this move in the context of the board.
 
         Returns:
@@ -146,13 +185,7 @@ class BoardMove(Move):
         if Move(self.text) not in piece.allowed_moves():
             return False
 
-        boardrules = BoardRules(self)
-        try:
-            self.enpassant = boardrules.enpassant
-        except AttributeError:
-            pass
-
-        if not boardrules.valid:
+        if not self.valid:
             return False
 
         castling = self.is_castling()
@@ -162,7 +195,7 @@ class BoardMove(Move):
             side = self.castle_side()
             newboard = self.board.castle(side, piece.color)
 
-        if newboard.is_check() == piece.color:
+        if newboard.is_check(color=piece.color) == piece.color:
             return False
 
         return newboard
